import Cookie from "universal-cookie";
import { request } from "../scripts/request.ts";
import { createContext, useContext, useState } from "react";

export interface Userinfo {
  isLoggedIn: boolean,
  displayName: string,
  profiles: any,
  meta: any
}

const defaultUserinfo: Userinfo = {
  isLoggedIn: false,
  displayName: "Citizen",
  profiles: null,
  meta: null
};

const UserinfoContext = createContext<any>({ userinfo: defaultUserinfo });

export function UserinfoProvider(props: any) {
  const [userinfo, setUserinfo] = useState<Userinfo>(structuredClone(defaultUserinfo));

  function getAtk(): string {
    return new Cookie().get("identity_auth_key");
  }

  function checkLoggedIn(): boolean {
    return new Cookie().get("identity_auth_key");
  }

  async function readProfiles() {
    if (!checkLoggedIn()) return;

    const res = await request("/api/users/me", {
      credentials: "include"
    });

    if (res.status !== 200) {
      clearUserinfo();
      window.location.reload();
    }

    const data = await res.json();

    setUserinfo({
      isLoggedIn: true,
      displayName: data["nick"],
      profiles: null,
      meta: data
    });
  }

  function clearUserinfo() {
    const cookies = document.cookie.split(";");
    for (let i = 0; i < cookies.length; i++) {
      const cookie = cookies[i];
      const eqPos = cookie.indexOf("=");
      const name = eqPos > -1 ? cookie.substring(0, eqPos) : cookie;
      document.cookie = name + "=;expires=Thu, 01 Jan 1970 00:00:00 GMT";
    }

    setUserinfo(defaultUserinfo);
  }

  return (
    <UserinfoContext.Provider value={{ userinfo, readProfiles, checkLoggedIn, getAtk, clearUserinfo }}>
      {props.children}
    </UserinfoContext.Provider>
  );
}

export function useUserinfo() {
  return useContext(UserinfoContext);
}